import ru.ifmo.testlib.Checker;
import ru.ifmo.testlib.InStream;
import ru.ifmo.testlib.Outcome;
import java.util.*;
import java.util.function.*;

import static ru.ifmo.testlib.Outcome.Type.*;

public class Check implements Checker {
    private static final int MAX_C = 1_000_000_000;
    private static final int[] dx = new int[]{1, -1, 0, 0};
    private static final int[] dy = new int[]{0, 0, 1, -1};

    public Outcome outcome(Outcome.Type type, final String format, final Object... args) {
        return new Outcome(type, String.format(format, args));
    }

    private long encode(int x, int y) {
        return x * (3L * MAX_C + 1) + y;
    }

    private long encode(int[] cell) {
        return encode(cell[0], cell[1]);
    }

    private void dfs(Set<Long> cells, Set<Long> used, int x, int y) {
        used.add(encode(x, y));
        for (int it = 0; it < 4; it++) {
            int nx = x + dx[it];
            int ny = y + dy[it];
            long code = encode(nx, ny);
            if (cells.contains(code) && !used.contains(code)) {
                dfs(cells, used, nx, ny);
            }
        }
    }

    private boolean isConnected(int[][] cells) {
        Set<Long> used = new HashSet<>();
        Set<Long> allCells = new HashSet<>();
        for (int[] cell : cells) {
            allCells.add(encode(cell));
        }
        dfs(allCells, used, cells[0][0], cells[0][1]);
        for (int[] cell : cells) {
            if (!used.contains(encode(cell))) {
                return false;
            }
        }
        return true;
    }

    private List<Long> foldv(int[][] cells, int x, boolean side) {
        //side == 1 if to the right, 0 if to the left
        List<Long> result = new ArrayList<>();
        Set<Long> added = new HashSet<>();
        for (int[] cell : cells) {
            if ((side && cell[0] >= x) || (!side && cell[0] < x)) {
                long code = encode(cell);
                if (!added.contains(code)) {
                    added.add(code);
                    result.add(code);
                }
            } else {
                long code = encode(2 * x - cell[0] - 1, cell[1]);
                if (!added.contains(code)) {
                    added.add(code);
                    result.add(code);
                }
            }
        }
        return result;
    }

    private List<Long> foldh(int[][] cells, int y, boolean side) {
        //side == 1 if up, 0 if down
        List<Long> result = new ArrayList<>();
        Set<Long> added = new HashSet<>();
        for (int[] cell : cells) {
            if ((side && cell[1] >= y) || (!side && cell[1] < y)) {
                long code = encode(cell);
                if (!added.contains(code)) {
                    added.add(code);
                    result.add(code);
                }
            } else {
                long code = encode(cell[0], 2 * y - cell[1] - 1);
                if (!added.contains(code)) {
                    added.add(code);
                    result.add(code);
                }
            }
        }
        return result;
    }

    private boolean isFoldCorrect(List<Long> fold, int[][] cells) {
        if (fold.size() != cells.length) {
            return false;
        }
        List<long[]> fs = new ArrayList<>();
        List<long[]> cs = new ArrayList<>();
        for (long f : fold) {
            long x = f / (3L * MAX_C + 1);
            long y = f % (3L * MAX_C + 1);
            fs.add(new long[]{x, y});
        }
        for (int[] c : cells) {
            cs.add(new long[]{c[0] * 1L, c[1] * 1L});
        }
        Collections.sort(fs, (a, b) -> {
            if (a[0] != b[0]) {
                return a[0] < b[0] ? -1 : 1;
            }
            return a[1] < b[1] ? -1 : (a[1] == b[1] ? 0 : 1);
        });
        Collections.sort(cs, (a, b) -> {
            if (a[0] != b[0]) {
                return a[0] < b[0] ? -1 : 1;
            }
            return a[1] < b[1] ? -1 : (a[1] == b[1] ? 0 : 1);
        });
        long nx = fs.get(0)[0] - cs.get(0)[0];
        long ny = fs.get(0)[1] - cs.get(0)[1];
        for (int i = 1; i < fs.size(); i++) {
            long[] f = fs.get(i);
            long[] c = cs.get(i);
            if (f[0] - c[0] != nx || f[1] - c[1] != ny) {
                return false;
            }
        }
        return true;
    }

    @Override
    public Outcome test(InStream inf, InStream ouf, InStream ans) {
        int tests = inf.nextInt();
        for (int testNum = 1; testNum <= tests; testNum++) {
            int n = inf.nextInt();
            int k = inf.nextInt();
            int[][] foldCells = new int[n][2];
            for (int i = 0; i < n; i++) {
                foldCells[i][0] = inf.nextInt() + MAX_C;
                foldCells[i][1] = inf.nextInt() + MAX_C;
            }
            String type = ouf.nextToken();
            int coordinate = ouf.nextInt();
            if (!type.equals("L") && !type.equals("R") &&
                !type.equals("U") && !type.equals("D")) {
                return outcome(WA, String.format("For test %d: unknown fold type: %s", testNum, type));
            }
            if (-MAX_C > coordinate || coordinate > MAX_C) {
                return outcome(WA, String.format("For test %d: fold coordinate should be in [%d, %d]", testNum, -MAX_C, MAX_C));
            }
            coordinate += MAX_C;
            int[][] cells = new int[k][2];
            Set<Long> used = new HashSet<>();
            for (int i = 0; i < k; i++) {
                cells[i][0] = ouf.nextInt();
                cells[i][1] = ouf.nextInt();
                if (-MAX_C > cells[i][0] || cells[i][0] > MAX_C ||
                    -MAX_C > cells[i][1] || cells[i][1] > MAX_C) {
                    return outcome(WA, String.format("For test %d: not all output cells are in [%d, %d]", testNum, -MAX_C, MAX_C));
                }
                cells[i][0] += MAX_C;
                cells[i][1] += MAX_C;
                long code = encode(cells[i][0], cells[i][1]);
                if (used.contains(code)) {
                    return outcome(WA, String.format("For test %d: output contains duplicate cells", testNum));
                }
                used.add(code);
            }
            if (!isConnected(cells)) {
                return outcome(WA, String.format("For test %d: cells in output are not connected", testNum));
            }
            if ((type.equals("L") && !isFoldCorrect(foldv(cells, coordinate, true), foldCells)) ||
                (type.equals("R") && !isFoldCorrect(foldv(cells, coordinate, false), foldCells)) ||
                (type.equals("D") && !isFoldCorrect(foldh(cells, coordinate, true), foldCells)) ||
                (type.equals("U") && !isFoldCorrect(foldh(cells, coordinate, false), foldCells))) {
                return outcome(WA, String.format("For test %d: folded output figure doesn't equal input figure", testNum));
            }
        }
        return outcome(OK, String.format("%d tests ok", tests));
    }
}
